(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('eventemitter3')) :
  typeof define === 'function' && define.amd ? define(['exports', 'react', 'eventemitter3'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['clean-state'] = {}, global.react, global.EventEmitter));
}(this, (function (exports, react, EventEmitter) { 'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  var EventEmitter__default = /*#__PURE__*/_interopDefaultLegacy(EventEmitter);

  function splitPropertyAndMethod(modules) {
      var rootState = {};
      var rootReducers = {};
      var rootEffects = {};
      Object.keys(modules).forEach(function (key) {
          var module = modules[key];
          rootState[key] = {};
          rootReducers[key] = {};
          rootEffects[key] = {};
          Object.assign(rootState[key], module.state);
          Object.assign(rootReducers[key], module.reducers);
          Object.assign(rootEffects[key], module.effects);
      });
      return { rootState: rootState, rootReducers: rootReducers, rootEffects: rootEffects };
  }

  var Container = (function () {
      function Container(modules) {
          var initialData = splitPropertyAndMethod(modules);
          this.rootState = initialData.rootState;
          this.rootReducers = initialData.rootReducers;
          this.rootEffects = initialData.rootEffects;
          this.emitter = new EventEmitter__default['default']();
      }
      Container.prototype.getRootEffects = function () {
          return this.rootEffects;
      };
      Container.prototype.getRootReducers = function () {
          return this.rootReducers;
      };
      Container.prototype.getRootState = function () {
          return this.rootState;
      };
      Container.prototype.getModule = function (namespace) {
          var _this = this;
          var combineModule = {};
          var assign = function (k) {
              var _a;
              var module = {
                  state: _this.rootState[k],
                  reducers: _this.rootReducers[k],
                  effects: _this.rootEffects[k]
              };
              Object.assign(combineModule, (_a = {}, _a[k] = module, _a));
          };
          if (Array.isArray(namespace)) {
              namespace.forEach(function (k) { return assign(k); });
          }
          else {
              assign(namespace);
          }
          return combineModule;
      };
      Container.prototype.getState = function (namespace) {
          var combineModule = this.getModule(namespace);
          return Object.keys(combineModule).reduce(function (result, key) {
              result[key] = combineModule[key].state;
              return result;
          }, {});
      };
      Container.prototype.setState = function (namespace, newState) {
          if (this.rootState[namespace]) {
              this.rootState[namespace] = newState;
          }
          this.trigger(namespace);
      };
      Container.prototype.addListener = function (namespace, listener) {
          var _this = this;
          if (Array.isArray(namespace)) {
              namespace.forEach(function (k) { return _this.emitter.on(k, listener); });
          }
          else {
              this.emitter.on(namespace, listener);
          }
      };
      Container.prototype.removeListener = function (namespace, listener) {
          var _this = this;
          if (Array.isArray(namespace)) {
              namespace.forEach(function (k) { return _this.emitter.removeListener(k, listener); });
          }
          else {
              this.emitter.removeListener(namespace, listener);
          }
      };
      Container.prototype.trigger = function (namespace) {
          this.emitter.emit(namespace);
      };
      return Container;
  }());

  var DISPATCH_TYPE = 'CS_DISPATCH_TYPE';
  var plugins = [];
  var bootstrap = function (modules) {
      var container = new Container(modules);
      var pluginEmitter = new EventEmitter__default['default']();
      var dispatch = function (nameAndMethod, payload) {
          var _a = nameAndMethod.split('/'), namespace = _a[0], methodName = _a[1];
          var combineModule = container.getModule(namespace);
          var _b = combineModule[namespace], state = _b.state, reducers = _b.reducers, effects = _b.effects;
          var rootState = container.getRootState();
          if (effects[methodName]) {
              pluginEmitter.emit(DISPATCH_TYPE, {
                  type: nameAndMethod,
                  payload: payload
              });
              return effects[methodName]({ state: state, payload: payload, rootState: rootState, dispatch: dispatch });
          }
          else if (reducers[methodName]) {
              var newState = reducers[methodName]({
                  state: state,
                  rootState: rootState,
                  payload: payload
              });
              container.setState(namespace, newState);
              pluginEmitter.emit(DISPATCH_TYPE, {
                  type: nameAndMethod,
                  payload: payload,
                  newState: newState
              });
          }
      };
      var injectFns = function (reducersOrEffects) {
          Object.keys(reducersOrEffects).forEach(function (key) {
              if (!dispatch[key])
                  dispatch[key] = {};
              var originFns = reducersOrEffects[key];
              var fns = {};
              Object.keys(originFns).forEach(function (fnKey) {
                  fns[fnKey] = function (payload) {
                      return dispatch(key + "/" + fnKey, payload);
                  };
              });
              Object.assign(dispatch[key], fns);
          });
      };
      function useModule(namespace) {
          var _a = react.useState({}), setState = _a[1];
          var setStateProxy = react.useCallback(function () { return setState({}); }, [setState]);
          react.useEffect(function () {
              container.addListener(namespace, setStateProxy);
              return function () { return container.removeListener(namespace, setStateProxy); };
          }, [namespace, setStateProxy]);
          return container.getState(namespace);
      }
      var rootReducers = container.getRootReducers();
      var rootEffects = container.getRootEffects();
      injectFns(rootReducers);
      injectFns(rootEffects);
      plugins.forEach(function (plugin) { return plugin(modules, pluginEmitter); });
      return { useModule: useModule, dispatch: dispatch };
  };
  bootstrap.addPlugin = function (plugin) {
      plugins.push(plugin);
  };
  bootstrap.DISPATCH_TYPE = DISPATCH_TYPE;

  var mixin = function (common, modules) {
      var keys = Object.keys(modules);
      keys.forEach(function (key) {
          var module = modules[key];
          module.state = module.state || {};
          module.reducers = module.reducers || {};
          module.effects = module.effects || {};
          if (common.state)
              Object.assign(module.state, common.state);
          if (common.reducers)
              Object.assign(module.reducers, common.reducers);
          if (common.effects)
              Object.assign(module.effects, common.effects);
      });
      return modules;
  };

  exports.default = bootstrap;
  exports.mixin = mixin;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=index.min.js.map
